import os
import time
# pip3 install pynput
# pip install pynput
from pynput import keyboard
from pynput.keyboard import Key

SCENE_WIDTH = 11
SCENE_HEIGHT = 15


class Position2D(object):
    def __init__(self, val_x, val_y, is_check_scene=True):
        super().__init__()
        self.__is_check_scene = is_check_scene
        self.__x = val_x
        self.__y = val_y

    @property
    def x(self): # get_x
        return self.__x

    @x.setter
    def x(self, val): # set_x
        if not self.__is_check_scene or 0 <= val < SCENE_WIDTH:
            self.__x = val

    @property
    def y(self): # get_y
        return self.__y

    @y.setter
    def y(self, val):  # set_y
        if not self.__is_check_scene or 0 <= val < SCENE_HEIGHT:
            self.__y = val


class Element(object):
    def __init__(self): # constructor
        super().__init__()
        self._position = Position2D(0, 0)
        self._char = '⬜️'

    @property
    def char(self):
        return self._char

    @property
    def position(self):
        return self._position

    @position.setter
    def position(self, pos):
        self._position = pos

    def draw(self, scene):
        pos_x = int(self.position.x)
        pos_y = int(self.position.y)
        scene[pos_y][pos_x] = self.char

    def update(self, delta_time):
        pass

    def check_collsion(self, other):
        is_collision = False
        if int(self.position.x) == int(other.position.x) and int(self.position.y) == int(other.position.y):
            is_collision = True
        return is_collision


class Wall(Element):
    def __init__(self, position): # constructor
        super().__init__()
        self._char = '🏛'
        self._position = position

# TODO explosion


class MovableElement(Element):
    def __init__(self): # constructor
        super().__init__()
        self._char = '🛸'
        self._speed = 1.0
        self._direction = Position2D(0, 0, is_check_scene=False)

    def update(self, delta_time):
        self.position.x += self._direction.x * self._speed * delta_time
        self.position.y += self._direction.y * self._speed * delta_time

    def left(self):
        self._direction.x = -1
        self._direction.y = 0

    def right(self):
        self._direction.x = 1
        self._direction.y = 0

    def up(self):
        self._direction.x = 0
        self._direction.y = -1

    def down(self):
        self._direction.x = 0
        self._direction.y = 1


class Player(MovableElement):
    def __init__(self):
        super().__init__()
        self._speed = 1.5
        self._char = '📤'

    # TODO fire_rocket


# TODO rocket 🔺


# TODO singleton GameState


is_game_running = True

# TODO look at player controls and location
pos_middle = Position2D(
    val_x=int(SCENE_WIDTH/2),
    val_y=SCENE_HEIGHT-1
)
player = Player()
player.position = pos_middle

elements = [
    player,
    Wall(position=Position2D(2, SCENE_HEIGHT-4)),
    Wall(position=Position2D(4, SCENE_HEIGHT-4)),
    Wall(position=Position2D(6, SCENE_HEIGHT-4)),
    Wall(position=Position2D(8, SCENE_HEIGHT-4))
]


# Get emojis:
# Windows Key + .
# Mac contrl + cmd + space

# main game loop
while is_game_running:
    cmd_clear = 'clear'
    if os.name == 'nt':
        cmd_clear = 'cls'
    os.system(cmd_clear)

    scene = []
    for i in range(SCENE_HEIGHT): # 0...14
        columns = []
        for j in range(SCENE_WIDTH):
             columns.append('⬛️')
        scene.append(columns)

    for element in elements:
        element.draw(scene)

    scene_lines = []
    for line in scene:
        str_line = ''.join(line)
        scene_lines.append(str_line)
    str_scene = '\n'.join(scene_lines)
    print('')
    print(str_scene)

    delay = 0.5 # seconds
    timestamp = time.time()

    with keyboard.Events() as events:
        key = events.get(delay)

        if key is not None: # key != null
            key_code = key.key
            if key_code == Key.left:
                player.left()
            elif key_code == Key.right: # else if
                player.right()
            elif key_code == Key.space:
                #TODO fire rocket
                pass
            elif key_code == Key.esc:
                is_game_running = False
        else:
            # TODO stop
            pass



    dt = delay - (time.time() - timestamp)
    if dt > 0:
        time.sleep(dt)

    dt = time.time() - timestamp
    for element in elements:
        element.update(dt)

    is_collision = False
    for i in range(len(elements)):
        for j in range(i+1, len(elements)):
            # TODO check type
            if elements[i].check_collsion(elements[j]):
                # TODO remove elements
                # TODO Explosion
                is_collision = True
                break
        if is_collision:
            break
